/*
 *  Copyright (C) 2002 Steve Harris, ECS, University of Southampton
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  $Id: okbc_pp.c,v 1.4 2002/11/28 17:59:30 theno23 Exp $
 */

#include <httpd.h>
#include <http_protocol.h>

#include "okbc.h"
#include "uri_escape.h"

void node_pp_sexp(request_rec *r, okbc_node *node)
{
	unsigned int i;

	if (!node) {
		printf("null");
		return;
	}
        switch (node->type) {
                case OKBC_LIST:
                        ap_rputs("(", r);
                        for (i = 0; i < node->data.list->length; i++) {
                                node_pp_sexp(r, node->data.list->items[i]);
				if (i < node->data.list->length - 1) {
					ap_rputs(" ", r);
				}
                        }
                        ap_rputs(")", r);
                        break;
                case OKBC_URI:
                        ap_rprintf(r, "\"%s\"", (char *)node->data.atom);
                        break;
                case OKBC_LITERAL:
                        ap_rprintf(r, "\"%s\"", (char *)node->data.atom);
                        break;
                case OKBC_SYMBOL:
                        ap_rprintf(r, "%s", (char *)node->data.atom);
                        break;
                case OKBC_BOOL:
                        ap_rprintf(r, "%s", (int)node->data.atom == 0 ? "false" : "true");
                        break;
                default:
                        ap_rprintf(r, "[%p]", node->data.atom);
                        break;
        }
}

void node_pp_xml_intl(request_rec *r, okbc_node *node, int depth);

void node_pp_xml(request_rec *r, okbc_node *node)
{
	ap_rputs("<?xml version=\"1.0\"?>\n", r);
	node_pp_xml_intl(r, node, 0);
}


void node_pp_xml_intl(request_rec *r, okbc_node *node, int depth)
{
	unsigned int i;
	char *tmp;

	if (!node) {
		printf("<!-- null -->");
		return;
	}
	for (i=0; i<depth; i++) {
		ap_rputs("  ", r);
	}
        switch (node->type) {
                case OKBC_LIST:
                        ap_rputs("<list>\n", r);
                        for (i = 0; i < node->data.list->length; i++) {
                                node_pp_xml_intl(r, node->data.list->items[i], depth+1);
                        }
			for (i=0; i<depth; i++) {
				ap_rputs("  ", r);
			}
                        ap_rputs("</list>\n", r);
                        break;
                case OKBC_URI:
			tmp = xml_escape((char *)node->data.atom);
                        ap_rprintf(r, "<item uri=\"%s\" />\n", tmp);
			free(tmp);
                        break;
                case OKBC_LITERAL:
			tmp = xml_escape((char *)node->data.atom);
                        ap_rprintf(r, "<item>%s</item>\n", tmp);
			free(tmp);
                        break;
                case OKBC_SYMBOL:
                        ap_rprintf(r, "<symbol>%s</symbol>\n", (char *)node->data.atom);
                        break;
                case OKBC_BOOL:
                        ap_rprintf(r, "<bool>%s</bool>\n", (int)node->data.atom == 0 ? "false" : "true");
                        break;
                default:
                        ap_rprintf(r, "<!-- unknown: %p -->", node->data.atom);
                        break;
        }
}
