#include "okbc_types.h"

okbc_node *okbc_all_connections();

okbc_node *okbc_allocate_frame_handle(const char *frame_name,
				      const char *frame_type,
				      const char *kb,
				      const char *frame_handle);

okbc_node *okbc_askable(const char *sentence, const char *kb,
			const char *value_selector, int kb_local_only_p);

okbc_node *okbc_ask(const char *query, const char *kb,
		    const char *reply_pattern, const char *inference_level,
		    int number_of_values, int error_p,
		    const char *where, const char *timeout,
		    const char *value_selector, int kb_local_only_p);

okbc_node *okbc_call_procedure(const char *procedure, const char *kb,
			       const char *arguments);

okbc_node *okbc_class_p(const char *thing, const char *kb,
			int kb_local_only_p);

okbc_node *okbc_coercible_to_frame_p(const char *thing, const char *kb,
				     int kb_local_only_p);

okbc_node *okbc_connection_p(const char *thing);

okbc_node *okbc_continuable_error_p(const char *thing);

okbc_node *okbc_current_kb();

okbc_node *okbc_decontextualize(const char *value,
				const char *from_context, const char *kb);

okbc_node *okbc_eql_in_kb(const char *arg0, const char *arg1,
			  const char *kb, int kb_local_only_p);

okbc_node *okbc_equal_in_kb(const char *arg0, const char *arg1,
			    const char *kb, int kb_local_only_p);

okbc_node *okbc_equalp_in_kb(const char *arg0, const char *arg1,
			     const char *kb, int kb_local_only_p);

okbc_node *okbc_establish_connection(const char *connection_type,
				     const char *initargs);

void okbc_expunge_kb(const char *kb_locator, const char *kb_type,
		     const char *connection, int error_p);

okbc_node *okbc_fetch(const char *enumerator,
		      int number_of_values);

okbc_node *okbc_follow_slot_chain(const char *frame,
				  const char *slot_chain, const char *kb,
				  const char *union_multiple_values,
				  const char *inference_level,
				  const char *value_selector,
				  int kb_local_only_p);

void okbc_free(const char *enumerator);

void okbc_goto_kb(const char *kb);

okbc_node *okbc_has_more(const char *enumerator);

okbc_node *okbc_individual_p(const char *thing, const char *kb,
			     int kb_local_only_p);

okbc_node *okbc_instance_of_p(const char *thing, const char *class,
			      const char *kb, const char *inference_level,
			      int kb_local_only_p);

okbc_node *okbc_local_connection();

okbc_node *okbc_meta_kb(const char *connection);

okbc_node *okbc_next(const char *enumerator);

okbc_node *okbc_okbc_condition_p(const char *thing);

okbc_node *okbc_openable_kbs(const char *kb_type, const char *connection,
			     const char *place);

okbc_node *okbc_open_kb(const char *kb_locator, const char *kb_type,
			const char *connection, int error_p);

void okbc_prefetch(const char *enumerator, int number_of_values,
		   const char *increment);

okbc_node *okbc_primitive_p(const char *class, const char *kb,
			    int kb_local_only_p);

okbc_node *okbc_print_frame(const char *frame, const char *kb,
			    const char *slots, const char *facets,
			    const char *stream,
			    const char *inference_level,
			    int number_of_values,
			    const char *value_selector,
			    int kb_local_only_p);

okbc_node *okbc_procedure_p(const char *thing);

void okbc_register_procedure(const char *name, const char *procedure,
			     const char *kb);

okbc_node *okbc_revert_kb(const char *kb, int error_p);

okbc_node *okbc_subclass_of_p(const char *subclass, const char *superclass,
			      const char *kb, const char *inference_level,
			      int kb_local_only_p);

okbc_node *okbc_superclass_of_p(const char *superclass,
				const char *subclass, const char *kb,
				const char *inference_level,
				int kb_local_only_p);

okbc_node *okbc_tellable(const char *sentence, const char *kb,
			 const char *value_selector, int kb_local_only_p);

void okbc_tell(const char *sentence, const char *kb, const char *frame,
	       const char *value_selector, int kb_local_only_p);

okbc_node *okbc_type_of_p(const char *class, const char *thing,
			  const char *kb, const char *inference_level,
			  int kb_local_only_p);

void okbc_unregister_procedure(const char *name, const char *kb);

okbc_node *okbc_untell(const char *sentence, const char *kb,
		       const char *frame, const char *value_selector,
		       int kb_local_only_p);

okbc_node *okbc_value_as_string(const char *value, const char *kb,
				const char *purpose, int pretty_p,
				int kb_local_only_p);

void okbc_attach_facet(const char *frame, const char *slot,
		       const char *facet, const char *kb,
		       const char *slot_type, int kb_local_only_p);

void okbc_attach_slot(const char *frame, const char *slot, const char *kb,
		      const char *slot_type, int kb_local_only_p);

void okbc_close_connection(const char *connection, int force_p,
			   int error_p);

void okbc_close_kb(const char *kb, int save_p);

okbc_node *okbc_copy_frame(const char *frame, const char *new_name,
			   const char *kb, const char *to_kb, int error_p,
			   const char *missing_frame_action,
			   const char *frame_handle_mapping_table,
			   int kb_local_only_p);

void okbc_copy_kb(const char *from_kb, const char *to_kb, int error_p,
		  const char *missing_frame_action, int kb_local_only_p);

void okbc_detach_facet(const char *frame, const char *slot,
		       const char *facet, const char *kb,
		       const char *slot_type, int kb_local_only_p);

void okbc_detach_slot(const char *frame, const char *slot, const char *kb,
		      const char *slot_type, int kb_local_only_p);

okbc_node *okbc_facet_has_value_p(const char *frame, const char *slot,
				  const char *facet, const char *kb,
				  const char *inference_level,
				  const char *slot_type,
				  const char *value_selector,
				  int kb_local_only_p);

okbc_node *okbc_facet_p(const char *thing, const char *kb,
			int kb_local_only_p);

okbc_node *okbc_frame_has_slot_p(const char *frame, const char *slot,
				 const char *kb,
				 const char *inference_level,
				 const char *slot_type,
				 int kb_local_only_p);

okbc_node *okbc_frame_in_kb_p(const char *thing, const char *kb,
			      int kb_local_only_p);

okbc_node *okbc_frs_independent_frame_handle(const char *frame,
					     const char *kb,
					     int kb_local_only_p);

okbc_node *okbc_frs_name(const char *kb_type, const char *connection);

okbc_node *okbc_kb_modified_p(const char *kb);

okbc_node *okbc_kb_p(const char *thing);

okbc_node *okbc_rename_facet(const char *facet, const char *new_name,
			     const char *kb, int kb_local_only_p);

okbc_node *okbc_rename_slot(const char *slot, const char *new_name,
			    const char *kb, int kb_local_only_p);

void okbc_replace_facet_value(const char *frame, const char *slot,
			      const char *facet, const char *old_value,
			      const char *new_value, const char *kb,
			      const char *test, const char *slot_type,
			      const char *value_selector,
			      int kb_local_only_p);

void okbc_replace_slot_value(const char *frame, const char *slot,
			     const char *old_value, const char *new_value,
			     const char *kb, const char *test,
			     const char *slot_type, const char *index,
			     const char *value_selector,
			     int kb_local_only_p);

okbc_node *okbc_save_kb(const char *kb, int error_p);

void okbc_save_kb_as(const char *new_name_or_locator, const char *kb);
